/*****************************************************************************
*                    Laser Plasma Interaction Simulator                      *
*                                    Version 1.0a                            *
*                                       2005.9.12                            *
*                          Copyright  Keita Ogawa                            * 
*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include <stdlib.h>

/*****************************************************************************
*                           Simulation Parameter                             *
*****************************************************************************/
#define dx      0.4                        /* x grid spacing                */
#define dy      0.4                        /* y grid spacing                */
#define dz      0.4                        /* z grid spacing                */
#define dt      0.1                        /* time step                     */

#define imx     356                        /* number of grid points (x)     */
#define jmx      74                        /* number of grid points (y)     */
#define kmx      74                        /* number of grid points (z)     */
#define nmx 5000000                        /* number of particles           */
#define tmx    4000                        /* time limit                    */

#define me      1.0                        /* mass of electron 9.1E-31      */
#define mi   1837.0                        /* mass of ion                   */
#define qi      1.0                        /* charge of ion    1.6E-19      */
#define qe     -1.0                        /* charge of electron            */

#define vtxe    0.04422895799              /* thermal velocity(x) of e      */
#define vtye    0.04422895799              /* thermal velocity(y) of e      */
#define vtze    0.04422895799              /* thermal velocity(z) of e      */
#define vdxe    0.0                        /* drift velocity(x) of electron */
#define vdye    0.0                        /* drift velocity(y) of electron */
#define vdze    0.0                        /* drift velocity(z) of electron */

#define vtxi    0.0009229902706            /* thermal velocity(x) of ion    */
#define vtyi    0.0009229902706            /* thermal velocity(y) of ion    */
#define vtzi    0.0009229902706            /* thermal velocity(z) of ion    */
#define vdxi    0.0                        /* drift velocity(x) of ion      */
#define vdyi    0.0                        /* drift velocity(y) of ion      */
#define vdzi    0.0                        /* drift velocity(z) of ion      */

#define Eo      2.866765509                /* intensity I_18 = 0.9062517423 */
                                           /*           I_19 = 2.866765509  */
                                           /*           I_20 = 9.065508526  */
#define tau     (1.77702978 *40.0)         /* a constant time               */
#define w       (5.927533309*10.0)         /* width of LASER 60.0           */
#define omeg    (1.0)                      /* laser freqency                */

#define no      (3.0)                      /* clitical density ratio        */
#define lp      (5.927533309*20.0)         /* layer of plasma  71.17724     */
#define lg      (5.927533309)              /* layer of gradient             */
#define dbp     (5.927533309*2.0)          /* distance by plazma wall 300.0 */
#define um      (5.927533309)              /* united 1um for output data    */ 

#define FBUF    50                         /* files buffer                  */
#define BEGIN   0                          /* record start time     (lap)   */
#define STEP    100                        /* interval output data  (lap)   */

/*****************************************************************************
*                           Useful Parameter definition                      *
*****************************************************************************/
#define hdx   (dx/2.0)                     /* half of dx                    */
#define hdy   (dy/2.0)                     /* half of dy                    */
#define hdz   (dz/2.0)                     /* half of dz                    */
#define hdt   (dt/2.0)                     /* half of dt                    */
#define qdt   (dt/4.0)                     /* quarter of dt                 */

#define Lx    (imx*dx)                     /* distance x of Simulation Area */
#define Ly    (jmx*dy)                     /* distance y of Simulation Area */
#define Lz    (kmx*dz)                     /* distance z of Simulation Area */
#define yo    (jmx/2.0*dy)                 /* center point of y direction   */
#define zo    (kmx/2.0*dz)                 /* center point of z direction   */

#define tg    (imx*jmx*kmx)

#define imm   (imx-1)
#define jmm   (jmx-1)
#define kmm   (kmx-1) 
#define imp   (imx+1)
#define jmp   (jmx+1)
#define kmp   (kmx+1) 

#define twp   (3.1415926535*2.0)           /* constant of normalize         */
#define llt   (3.141592654*tau/dt)

#ifdef gradient
#define tlp   (lg+lp)                      /* total layer plazma wall       */
#else
#define tlp   (lp)
#endif

/*****************************************************************************
*                         Environment of Simulation                          *
*****************************************************************************/
#define laser                              /* irradiation of laser          */
#define _gradient                          /* gradient at plazma wall       */
#define DEBUG                              /* for DEBUG mode                */
#define system_setup                       /* output system setup file      */
#define project_laser
#define project_density

#define particle_phase                     
#define _particle_density_loz
#define _particle_density_pm3d
#define particle_statistics
#define particle_distribution
#define particle_kinetic

#define energy_total                   
#define energy_current
#define energy_spectrum
#define energy_gain                        /* output energy gain            */
#define _energy_gain_hourly

#define _field_absorption
#define _field_current
#define _field_electric                     
#define _field_magnetic
#define field_RMS                          /* output root mean square       */
#define field_magnetic_hourly
#define field_current_hourly
#define particle_density_loz_hourly

#define _thermalflux             
#define generated_magnetic

/*****************************************************************************
*                         Simulation Properties                              *
*****************************************************************************/
static double    m[2];                     /* mass of particle              */
static double    q[2];                     /* charge of particle            */
static double  vtx[2];                     /* thermal Velocity of particle  */
static double  vty[2];                     /* thermal Velocity of particle  */
static double  vtz[2];                     /* thermal Velocity of particle  */
static double  vdx[2];                     /* drift velocity(x) of particle */
static double  vdy[2];                     /* drift velocity(y) of particle */
static double  vdz[2];                     /* drift velocity(z) of particle */
static double  ave[3];                     /* average Velocity of particle  */ 
static double  dev[3];                     /* thermal Velocity of particle  */

static double    x[2][nmx];                /* position (x) of particle      */
static double    y[2][nmx];                /* position (y) of particle      */
static double    z[2][nmx];                /* position (z) of particle      */
static double   hx[2][nmx];                /* half position (x) of particle */
static double   hy[2][nmx];                /* half position (y) of particle */
static double   hz[2][nmx];                /* half position (z) of particle */
static double   vx[2][nmx];                /* velocity (x) of particle      */
static double   vy[2][nmx];                /* velocity (y) of particle      */
static double   vz[2][nmx];                /* velocity (z) of particle      */
static double  hvx[2][nmx];                /* velocity (x) of particle      */
static double  Epx[2][nmx];                /* bound electric field (x)      */
static double  Epy[2][nmx];                /* bound electric field (y)      */
static double  Epz[2][nmx];                /* bound electric field (z)      */
static double  Bpx[2][nmx];                /* bound magnetic field (x)      */
static double  Bpy[2][nmx];                /* bound magnetic field (y)      */
static double  Bpz[2][nmx];                /* bound magnetic field (z)      */
static double  enz[nmx];

static double  Ex[imx+2][jmx+1][kmx+1];    /* electric field (x)            */
static double  Ey[imx+1][jmx+2][kmx+1];    /* electric field (y)            */
static double  Ez[imx+1][jmx+1][kmx+2];    /* electric field (z)            */
static double  Bx[imx+1][jmx+2][kmx+2];    /* magnetic field (x)            */
static double  By[imx+2][jmx+1][kmx+2];    /* magnetic field (y)            */
static double  Bz[imx+2][jmx+2][kmx+1];    /* magnetic field (z)            */
static double  jx[imx+2][jmx+1][kmx+1];    /* current density (x)           */
static double  jy[imx+1][jmx+2][kmx+1];    /* current density (y)           */
static double  jz[imx+1][jmx+1][kmx+2];    /* current density (z)           */
static double tmp[imx+2][jmx+2][kmx+2];    /* temp                          */

#ifdef field_magnetic_hourly
static double Bhx[imx+1][jmx+1][kmx+1];    /* magnetic field (x)            */
static double Bhy[imx+1][jmx+1][kmx+1];    /* magnetic field (y)            */
static double Bhz[imx+1][jmx+1][kmx+1];    /* magnetic field (z)            */
#endif
#ifdef field_current_hourly
static double jhx[imx+1][jmx+1][kmx+1];    /* current  field (x)            */
static double jhy[imx+1][jmx+1][kmx+1];    /* current  field (y)            */
static double jhz[imx+1][jmx+1][kmx+1];    /* current  field (z)            */
#endif

static double vv[9];                       /* areas for PIC-method          */
static double np;                          /* plasma density                */
static double norm;                        /* normalize density             */

static char   fname[FBUF];                 /* filename variable             */
static int    i,j,k,lap = 0;               /* counter  variable             */
static int    a,b,c;                       /* neighborhood (x,y,z)          */

/*****************************************************************************
*                         Function Declarations                              *
*****************************************************************************/
void ini_bunch                 (void);     /* initial particle & field      */
void ini_c                     (void);     /* initial constants             */
void ini_p                     (void);     /* initial paticles              */
void ini_v                     (void);     /* initial velocity              */

void cal_B                   (double);     /* calculate magnetic field      */
void cal_E                   (double);     /* calculate electric field      */
void cal_v                     (void);     /* calculate velocity            */
void cal_p                   (double);     /* calculate particle            */
void cal_j                     (void);     /* calculate current density     */
void cal_Laser                 (void);     /* calculate LASER               */

void out_bunch                 (void);
void out_field_electric        (void);     /* output E&B field              */
void out_field_magnetic        (void);     /* output E&B field              */
void out_field_current         (void);     /* output current value          */
void out_field_RMS             (void);
void out_field_magnetic_hourly (void);
void out_field_current_hourly  (void);
void out_field_absorption      (void);

void out_particle_density      (void);     /* output density                */
void out_particle_phase_pm3d   (void);
void out_particle_kinetic      (void);
void out_particle_phase        (void);
void out_particle_maxwell      (void);
void out_particle_density_loz  (void);
void out_particle_density_loz_hourly(void);
void out_particle_density_pm3d (void);
void out_particle_lorentz      (void);
void out_particle_distribution (void);
void out_statistics_table      (void);
void out_particle_statistics   (void);

void out_thermalflux           (void);
void out_system_setup          (void);     /* output setup file             */

void out_energy_total          (void);     /* output Total Energy           */
void out_energy_current        (void);     /* output current energy         */
void out_energy_spectrum       (void);     /* output energy spectrum        */
void out_energy_gain           (void);

void out_generated_magnetic    (void);

void out_project_laser         (void);
void out_project_density       (void);

void dis_EpBp                  (void);     /* distribute Ep & Bp            */
void dis_cube                  (double,double,double,double,double,double);

double rnd                   (double);     /* random function               */
double dt_dx                 (double);     /* operator of absorption        */
double grad           (double,double);     /* gradient                      */
double vav     (double,double,double);     /* vector abs value              */
double lorentz (double,double,double);     /* lorentz factor                */

int  nbh              (double,double);

/*****************************************************************************
*                         Simulation Programs                                *
*****************************************************************************/
int main()
{
  ini_bunch();
  out_bunch();

  printf("Now simulating ...\n");
  for(lap=1;lap<=tmx;lap++)
    {
#ifdef laser
      cal_Laser();
#endif
      dis_EpBp();
      cal_v();

      cal_p(dt);
      cal_j();

      cal_B(qdt);
      cal_E(hdt);
      cal_B(hdt);
      cal_E(hdt);
      cal_B(qdt);

      out_bunch();
      if(lap % STEP == 0)
	printf("now process ... %3.0f %%\n",
	       (double)(lap)/(double)(tmx)*100);
    }
  
  printf("Complated \a\n");
  return 0;
}

/*****************************************************************************
*                         Initialzation parameters                           *
*****************************************************************************/
void ini_bunch(void)
{
  printf("Now initialzation ...\n");

  ini_c();
  ini_p();
  ini_v();

#ifdef system_setup
  out_system_setup();
#endif

  printf("End initialzation \n");
}

void ini_c(void)
{
  vtx[0] = vtxe; vtx[1] = vtxi;
  vty[0] = vtye; vty[1] = vtyi;
  vtz[0] = vtze; vtz[1] = vtzi;

  vdx[0] = vdxe; vdx[1] = vdxi;
  vdy[0] = vdye; vdy[1] = vdyi;
  vdz[0] = vdze; vdz[1] = vdzi;
  
  m[0] = me;     m[1] = mi;
  q[0] = qe;     q[1] = qi;
  
  vv[0] = dx*dy*dz;
  
  for(j=0;j<3;j++)
    ave[j] = dev[j] = 0.0;
  
  for(i=0;i<2;i++)
    for(j=0;j<nmx;j++)
      x[i][j] =  y[i][j] =  z[i][j] = 0.0;
  
  for(i=0;i<2;i++)
    for(j=0;j<nmx;j++)
      hx[i][j] = hy[i][j] = hz[i][j] = 0.0;
  
  for(i=0;i<=imp;i++)
    for(j=0;j<=jmx;j++)
      for(k=0;k<=kmx;k++)
	Ex[i][j][k] = jx[i][j][k] = 0.0;
  
  for(i=0;i<=imx;i++)
    for(j=0;j<=jmp;j++)
      for(k=0;k<=kmx;k++)
	Ey[i][j][k] = jy[i][j][k] = 0.0;
  
  for(i=0;i<=imx;i++)
    for(j=0;j<=jmx;j++)
      for(k=0;k<=kmp;k++)
	Ez[i][j][k] = jz[i][j][k] = 0.0;
  
  for(i=0;i<=imx;i++)
    for(j=0;j<=jmp;j++)
      for(k=0;k<=kmp;k++)
	Bx[i][j][k] = 0.0;
  
  for(i=0;i<=imp;i++)
    for(j=0;j<=jmx;j++)
      for(k=0;k<=kmp;k++)
	By[i][j][k] = 0.0;
  
  for(i=0;i<=imp;i++)
    for(j=0;j<=jmp;j++)
      for(k=0;k<=kmx;k++)
	Bz[i][j][k] = 0.0;
}

void ini_v(void)
{
  double fvx,fvy,fvz;

  for(i=0;i<2;i++)
    for(j=0;j<nmx;j++)
      {
	do{
	  vx[i][j] = vy[i][j] = vz[i][j] = 0.0;
	  
	  for(k=0;k<12;k++)
	    {
	      vx[i][j] += rnd(1);
	      vy[i][j] += rnd(1);
	      vz[i][j] += rnd(1);
	    }
	  
	  vx[i][j] -= 6.0;
	  vy[i][j] -= 6.0;
	  vz[i][j] -= 6.0;
	  
	  vx[i][j] = vdx[i] + vtx[i]*vx[i][j];
	  vy[i][j] = vdy[i] + vty[i]*vy[i][j];
	  vz[i][j] = vdz[i] + vtz[i]*vz[i][j];
	  
	  fvx = fabs(vx[i][j]);
	  fvy = fabs(vy[i][j]);
	  fvz = fabs(vz[i][j]);
	  
	}while(fvx >= 1.0 || fvy >= 1.0 || fvz >= 1.0);
      }
}

void ini_p(void)
{
#ifdef gradient
  double psg = lg / 2.0;    /* area of gradient        */
  double psf = lp;          /* area of flat            */
  double pss = psg+psf;     /* area of plasma          */
  double Ng  = psg*nmx/pss; /* number of gradient area */
  double Nf  = psf*nmx/pss; /* number of flat area     */
  
  /* layer of gradient plasma */
  for(i=0;i<2;i++)
    for(j=0;j<Ng;j++)
      x[i][j] = grad(lg,dbp-lg);
  
  /* layer of flat plazma  */
  for(i=0;i<2;i++)
    for(j=Ng;j<nmx;j++)
      x[i][j] = rnd(lp)+dbp;

  np   = Nf/((lp/dx)*(double)(jmx*kmx));
#else
  /* all layer of flat plazma  */
  for(i=0;i<2;i++)
    for(j=0;j<nmx;j++)
      x[i][j] = rnd(lp)+dbp;

  np = (double)(nmx)/((tlp/dx)*(double)(jmx*kmx));
#endif
  for(i=0;i<2;i++)
    for(j=0;j<nmx;j++)
      y[i][j] = rnd(Ly);
  
  for(i=0;i<2;i++)
    for(j=0;j<nmx;j++)
      z[i][j] = rnd(Lz);

  norm = no/np;
}

/*****************************************************************************
*                         Simulation Functions                               *
*****************************************************************************/
void cal_p(double ddt)
{
  for(i=0;i<2;i++)
    for(j=0;j<nmx;j++)
      {
        hx[i][j] = x[i][j];
        hy[i][j] = y[i][j];
        hz[i][j] = z[i][j];

	hvx[i][j]=vx[i][j];

        x[i][j] += vx[i][j]*ddt;
        y[i][j] += vy[i][j]*ddt;
        z[i][j] += vz[i][j]*ddt;

        hx[i][j] = (hx[i][j] + x[i][j])/2.0;
        hy[i][j] = (hy[i][j] + y[i][j])/2.0;
        hz[i][j] = (hz[i][j] + z[i][j])/2.0;
	
        /* Boundary conditions of particles */
	if( x[i][j] < 0.0){  x[i][j]*= -1.0;             vx[i][j] *= -1.0; }
        if( x[i][j] >= Lx){  x[i][j] = 2.0*Lx -x[i][j];  vx[i][j] *= -1.0; }
	if(hx[i][j] < 0.0){ hx[i][j]*= -1.0;            hvx[i][j] *= -1.0; }
        if(hx[i][j] >= Lx){ hx[i][j] = 2.0*Lx-hx[i][j]; hvx[i][j] *= -1.0; }
	
        if( y[i][j] < 0.0){  y[i][j] += Ly; }
        if( y[i][j] >= Ly){  y[i][j] -= Ly; }
        if(hy[i][j] < 0.0){ hy[i][j] += Ly; }
        if(hy[i][j] >= Ly){ hy[i][j] -= Ly; }

        if( z[i][j] < 0.0){  z[i][j] += Lz; }
        if( z[i][j] >= Lz){  z[i][j] -= Lz; }
        if(hz[i][j] < 0.0){ hz[i][j] += Lz; }
        if(hz[i][j] >= Lz){ hz[i][j] -= Lz; }
      }
}

void cal_v(void)
{
  double vxm,vxd,vxi,bxg;
  double vym,vyd,vyi,byg;
  double vzm,vzd,vzi,bzg;
  double qmt,Loz;

  /* by Buneman-Bris Method */
  for(i=0;i<2;i++)
    for(j=0;j<nmx;j++)
      {
        qmt = q[i]/m[i]*hdt;
        /* v- */
        Loz = 1.0/sqrt(1.0 - vav(vx[i][j],vy[i][j],vz[i][j]));
        vxm = Loz*vx[i][j] + qmt*Epx[i][j];
        vym = Loz*vy[i][j] + qmt*Epy[i][j];
        vzm = Loz*vz[i][j] + qmt*Epz[i][j];

        /* v' */
        Loz = 1.0/sqrt(1.0 + vav(vxm,vym,vzm));
        bxg = qmt*Loz* Bpx[i][j];
        byg = qmt*Loz* Bpy[i][j];
        bzg = qmt*Loz* Bpz[i][j];

        vxd = vxm + (vym*bzg - vzm*byg);
        vyd = vym + (vzm*bxg - vxm*bzg);
        vzd = vzm + (vxm*byg - vym*bxg);

        /* v+ */
        /* Loz = 2.0/sqrt(1.0 + vav(bxg,byg,bzg)); */
	Loz = 2.0/(1.0 + vav(bxg,byg,bzg));
        vxi = vxm + (vyd*Loz*bzg - vzd*Loz*byg) + qmt*Epx[i][j];
        vyi = vym + (vzd*Loz*bxg - vxd*Loz*bzg) + qmt*Epy[i][j];
        vzi = vzm + (vxd*Loz*byg - vyd*Loz*bxg) + qmt*Epz[i][j];

        /* vn */
        Loz = 1.0/sqrt(1.0 + vav(vxi,vyi,vzi));
        vx[i][j] = Loz*vxi;
        vy[i][j] = Loz*vyi;
        vz[i][j] = Loz*vzi;
      }
}

void cal_B(double ddt)
{
  /* Calculation of Bx */
  /*
  for(i=0;i<=imx;i++)
    for(j=0;j<=jmp;j++)
      for(k=0;k<=kmp;k++)
	tmp[i][j][k] = Bx[i][j][k];
  */
  for(i=0;i<=imx;i++)
    for(j=1;j<=jmx;j++)
      for(k=1;k<=kmx;k++)
        Bx[i][j][k] +=
	  ((Ey[i][j][k]-Ey[i][j][k-1])/dz-(Ez[i][j][k]-Ez[i][j-1][k])/dy)*ddt;
  /*
  for(k=1;k<=kmx;k++)
    for(j=1;j<=jmx;j++)
      {
	Bx[  0][j][k]=tmp[  1][j][k]+dt_dx(ddt)*(Bx[  1][j][k]-tmp[  0][j][k]);
	Bx[imx][j][k]=tmp[imm][j][k]+dt_dx(ddt)*(Bx[imm][j][k]-tmp[imx][j][k]);
      }
  */
  
  /* Calculation of By */
  for(i=0;i<=imp;i++)
    for(j=0;j<=jmx;j++)
      for(k=0;k<=kmp;k++)
	tmp[i][j][k] = By[i][j][k];
  
  for(i=1;i<=imx;i++)
    for(j=0;j<=jmx;j++)
      for(k=1;k<=kmx;k++)
        By[i][j][k] +=
	  ((Ez[i][j][k]-Ez[i-1][j][k])/dx-(Ex[i][j][k]-Ex[i][j][k-1])/dz)*ddt;

  for(k=1;k<=kmx;k++)
    for(j=0;j<=jmx;j++)
      {
	By[  0][j][k]=tmp[  1][j][k]+dt_dx(ddt)*(By[  1][j][k]-tmp[  0][j][k]);
	By[imx][j][k]=tmp[imm][j][k]+dt_dx(ddt)*(By[imm][j][k]-tmp[imx][j][k]);
      }

  /* Calculation of Bz */
  for(i=0;i<=imp;i++)
    for(j=0;j<=jmp;j++)
      for(k=0;k<=kmx;k++)
	tmp[i][j][k] = Bz[i][j][k];

  for(i=1;i<=imx;i++)
    for(j=1;j<=jmx;j++)
      for(k=0;k<=kmx;k++)
        Bz[i][j][k] +=
	  ((Ex[i][j][k]-Ex[i][j-1][k])/dy-(Ey[i][j][k]-Ey[i-1][j][k])/dx)*ddt;

  for(k=0;k<=kmx;k++)
    for(j=1;j<=jmx;j++)
      {
	Bz[  0][j][k]=tmp[  1][j][k]+dt_dx(ddt)*(Bz[  1][j][k]-tmp[  0][j][k]);
	Bz[imx][j][k]=tmp[imm][j][k]+dt_dx(ddt)*(Bz[imm][j][k]-tmp[imx][j][k]);
      }

  /* Boundary conditions of magnetic field */
  for(i=0;i<=imx;i++)
    {
      for(j=1;j<=jmx;j++)
	{
	  Bx[i][j][kmp] = Bx[i][j][  1];
	  Bx[i][j][  0] = Bx[i][j][kmx];
	}
      for(k=1;k<=kmx;k++)
	{
	  Bx[i][  0][k] = Bx[i][jmx][k];
	  Bx[i][jmp][k] = Bx[i][  1][k];
	}
      
      Bx[i][  0][  0] = Bx[i][jmx][kmx];
      Bx[i][jmp][  0] = Bx[i][  0][kmx];
      Bx[i][  0][kmp] = Bx[i][jmx][  1];
      Bx[i][jmp][kmp] = Bx[i][  1][  1];
    }
  
  for(i=1;i<=imx;i++)
    {
      for(j=0;j<=jmx;j++)
	{
	  By[i][j][kmp] = By[i][j][  1];
	  By[i][j][  0] = By[i][j][kmx];
	}
    }

  for(i=1;i<=imx;i++)
    {
      for(k=0;k<=kmx;k++)
	{
	  Bz[i][jmp][k] = Bz[i][  1][k];
	  Bz[i][  0][k] = Bz[i][jmx][k];
	}
    }
}

void cal_E(double ddt)
{
  /* Calculation of Ex */
  /*
  for(i=0;i<=imp;i++)
    for(j=0;j<=jmx;j++)
      for(k=0;k<=kmx;k++)
	tmp[i][j][k] = Ex[i][j][k];
  */
  for(i=1;i<=imx;i++)
    for(j=0;j<=jmx;j++)
      for(k=0;k<=kmx;k++)
        Ex[i][j][k] += ((Bz[i][j+1][k]-Bz[i][j][k])/dy
			-(By[i][j][k+1]-By[i][j][k])/dz-jx[i][j][k])*ddt;
  /*
  for(k=0;k<=kmx;k++)
    for(j=0;j<=jmx;j++)
      {
	Ex[  0][j][k]=tmp[  1][j][k]+dt_dx(ddt)*(Ex[  1][j][k]-tmp[  0][j][k]);
	Ex[imx][j][k]=tmp[imm][j][k]+dt_dx(ddt)*(Ex[imm][j][k]-tmp[imx][j][k]);
      }
  */
  /* Calculation of Ey */
  for(i=0;i<=imx;i++)
    for(j=0;j<=jmp;j++)
      for(k=0;k<=kmx;k++)
	tmp[i][j][k] = Ey[i][j][k];

  for(i=0;i<=imx;i++)
    for(j=1;j<=jmx;j++)
      for(k=0;k<=kmx;k++)
        Ey[i][j][k] += ((Bx[i][j][k+1]-Bx[i][j][k])/dz
			-(Bz[i+1][j][k]-Bz[i][j][k])/dx-jy[i][j][k])*ddt;
  for(k=0;k<=kmx;k++)
    for(j=1;j<=jmx;j++)
      {
	Ey[  0][j][k]=tmp[  1][j][k]+dt_dx(ddt)*(Ey[  1][j][k]-tmp[  0][j][k]);
	Ey[imx][j][k]=tmp[imm][j][k]+dt_dx(ddt)*(Ey[imm][j][k]-tmp[imx][j][k]);
      }

  /* Calculation of Ez */
  for(i=0;i<=imx;i++)
    for(j=0;j<=jmx;j++)
      for(k=0;k<=kmp;k++)
	tmp[i][j][k] = Ez[i][j][k];

  for(i=0;i<=imx;i++)
    for(j=0;j<=jmx;j++)
      for(k=1;k<=kmx;k++)
        Ez[i][j][k] += ((By[i+1][j][k]-By[i][j][k])/dx
			-(Bx[i][j+1][k]-Bx[i][j][k])/dy-jz[i][j][k])*ddt;
  for(k=1;k<=kmx;k++)
    for(j=0;j<=jmx;j++)
      {
	Ez[  0][j][k]=tmp[  1][j][k]+dt_dx(ddt)*(Ez[  1][j][k]-tmp[  0][j][k]);
	Ez[imx][j][k]=tmp[imm][j][k]+dt_dx(ddt)*(Ez[imm][j][k]-tmp[imx][j][k]);
      }

  /* Boundary conditions of electric field */
  for(i=0;i<=imx;i++)
    {
      for(j=0;j<=jmx;j++)
	{
	  Ez[i][j][  0] = Ez[i][j][kmx];
	  Ez[i][j][kmp] = Ez[i][j][  1];
	}
      for(k=0;k<=kmx;k++)
	{
	  Ey[i][  0][k] = Ey[i][jmx][k];
	  Ey[i][jmp][k] = Ey[i][  1][k];
	}
    }
}

void dis_EpBp(void)
{
  double cb[9];
  
  for(i=0;i<2;i++)
    for(j=0;j<nmx;j++)
      {
        /* Distribution of magnetic fields */
        dis_cube(x[i][j],y[i][j],z[i][j],0,hdy,hdz);
        cb[1] = Bx[a  ][b  ][c  ]; cb[2] = Bx[a+1][b  ][c  ];
        cb[3] = Bx[a+1][b+1][c  ]; cb[4] = Bx[a  ][b+1][c  ];
        cb[5] = Bx[a  ][b  ][c+1]; cb[6] = Bx[a+1][b  ][c+1];
        cb[7] = Bx[a+1][b+1][c+1]; cb[8] = Bx[a  ][b+1][c+1];
	
        Bpx[i][j] = (cb[1]*vv[7]+cb[2]*vv[8]+cb[4]*vv[6]+cb[3]*vv[5]+
                     cb[5]*vv[3]+cb[6]*vv[4]+cb[8]*vv[2]+cb[7]*vv[1])/vv[0];

        dis_cube(x[i][j],y[i][j],z[i][j],hdx,0,hdz);
        cb[1] = By[a  ][b  ][c  ]; cb[2] = By[a+1][b  ][c  ];
        cb[3] = By[a+1][b+1][c  ]; cb[4] = By[a  ][b+1][c  ];
        cb[5] = By[a  ][b  ][c+1]; cb[6] = By[a+1][b  ][c+1];
        cb[7] = By[a+1][b+1][c+1]; cb[8] = By[a  ][b+1][c+1];

        Bpy[i][j] = (cb[1]*vv[7]+cb[2]*vv[8]+cb[4]*vv[6]+cb[3]*vv[5]+
                     cb[5]*vv[3]+cb[6]*vv[4]+cb[8]*vv[2]+cb[7]*vv[1])/vv[0];

        dis_cube(x[i][j],y[i][j],z[i][j],hdx,hdy,0);
        cb[1] = Bz[a  ][b  ][c  ]; cb[2] = Bz[a+1][b  ][c  ];
        cb[3] = Bz[a+1][b+1][c  ]; cb[4] = Bz[a  ][b+1][c  ];
        cb[5] = Bz[a  ][b  ][c+1]; cb[6] = Bz[a+1][b  ][c+1];
        cb[7] = Bz[a+1][b+1][c+1]; cb[8] = Bz[a  ][b+1][c+1];

        Bpz[i][j] = (cb[1]*vv[7]+cb[2]*vv[8]+cb[4]*vv[6]+cb[3]*vv[5]+
                     cb[5]*vv[3]+cb[6]*vv[4]+cb[8]*vv[2]+cb[7]*vv[1])/vv[0];

        /* Distribution of electric fields */
        dis_cube(x[i][j],y[i][j],z[i][j],hdx,0,0);
        cb[1] = Ex[a  ][b  ][c  ]; cb[2] = Ex[a+1][b  ][c  ];
        cb[3] = Ex[a+1][b+1][c  ]; cb[4] = Ex[a  ][b+1][c  ];
        cb[5] = Ex[a  ][b  ][c+1]; cb[6] = Ex[a+1][b  ][c+1];
        cb[7] = Ex[a+1][b+1][c+1]; cb[8] = Ex[a  ][b+1][c+1];

        Epx[i][j] = (cb[1]*vv[7]+cb[2]*vv[8]+cb[4]*vv[6]+cb[3]*vv[5]+
                     cb[5]*vv[3]+cb[6]*vv[4]+cb[8]*vv[2]+cb[7]*vv[1])/vv[0];

        dis_cube(x[i][j],y[i][j],z[i][j],0,hdy,0);
        cb[1] = Ey[a  ][b  ][c  ]; cb[2] = Ey[a+1][b  ][c  ];
        cb[3] = Ey[a+1][b+1][c  ]; cb[4] = Ey[a  ][b+1][c  ];
        cb[5] = Ey[a  ][b  ][c+1]; cb[6] = Ey[a+1][b  ][c+1];
        cb[7] = Ey[a+1][b+1][c+1]; cb[8] = Ey[a  ][b+1][c+1];

        Epy[i][j] = (cb[1]*vv[7]+cb[2]*vv[8]+cb[4]*vv[6]+cb[3]*vv[5]+
                     cb[5]*vv[3]+cb[6]*vv[4]+cb[8]*vv[2]+cb[7]*vv[1])/vv[0];

        dis_cube(x[i][j],y[i][j],z[i][j],0,0,hdz);
        cb[1] = Ez[a  ][b  ][c  ]; cb[2] = Ez[a+1][b  ][c  ];
        cb[3] = Ez[a+1][b+1][c  ]; cb[4] = Ez[a  ][b+1][c  ];
        cb[5] = Ez[a  ][b  ][c+1]; cb[6] = Ez[a+1][b  ][c+1];
        cb[7] = Ez[a+1][b+1][c+1]; cb[8] = Ez[a  ][b+1][c+1];

        Epz[i][j] = (cb[1]*vv[7]+cb[2]*vv[8]+cb[4]*vv[6]+cb[3]*vv[5]+
                     cb[5]*vv[3]+cb[6]*vv[4]+cb[8]*vv[2]+cb[7]*vv[1])/vv[0];
      }
}

void dis_cube(double xx,double yy,double zz,double ddx,double ddy,double ddz)
{  
  double lx[2],ly[2],lz[2];

  a = (int)((xx + ddx)/dx);
  b = (int)((yy + ddy)/dy);
  c = (int)((zz + ddz)/dz);

  lx[0] = (xx + ddx)-((double)(a)*dx);
  ly[0] = (yy + ddy)-((double)(b)*dy);
  lz[0] = (zz + ddz)-((double)(c)*dz);

  lx[1] = dx - lx[0];
  ly[1] = dy - ly[0];
  lz[1] = dz - lz[0];

  vv[1] = lx[0] * ly[0] * lz[0];
  vv[2] = lx[1] * ly[0] * lz[0];
  vv[3] = lx[0] * ly[1] * lz[0];
  vv[4] = lx[1] * ly[1] * lz[0];
  vv[5] = lx[0] * ly[0] * lz[1];
  vv[6] = lx[1] * ly[0] * lz[1];
  vv[7] = lx[0] * ly[1] * lz[1];
  vv[8] = lx[1] * ly[1] * lz[1];

  /* vv = v1+v2+vv[3]+vv[4]+v5+v6+v7+vv[8];*/
}

void cal_Laser(void)
{
  double t  = lap*dt;
  double cyo = jmx/2.0;
  double czo = kmx/2.0;

  if(lap <= (int)(llt)){
    for(k=0;k<=kmx;k++)
      for(j=1;j<=jmx;j++)
	{
	  Ey[0][j][k] = Eo*sin(t/tau)*sin(t/tau)*sin(-omeg*t)*
	    exp(-(j-cyo)*dy*(j-cyo)*dy/(w*w))*
	    exp(-(k-czo)*dz*(k-czo)*dz/(w*w));
	}
  }
}

void cal_j(void)
{
  for(i=0;i<=imp;i++)
    for(j=0;j<=jmx;j++)
      for(k=0;k<=kmx;k++)
        jx[i][j][k] = 0.0;

  for(i=0;i<=imx;i++)
    for(j=0;j<=jmp;j++)
      for(k=0;k<=kmx;k++)
        jy[i][j][k] = 0.0;

  for(i=0;i<=imx;i++)
    for(j=0;j<=jmx;j++)
      for(k=0;k<=kmp;k++)
        jz[i][j][k] = 0.0;

  for(i=0;i<2;i++)
    for(j=0;j<nmx;j++)
      {
        dis_cube(hx[i][j],hy[i][j],hz[i][j],hdx,0,0);
	
        jx[a  ][b  ][c  ] += q[i]*hvx[i][j]*(vv[7]/vv[0])*norm;
        jx[a+1][b  ][c  ] += q[i]*hvx[i][j]*(vv[8]/vv[0])*norm;
        jx[a  ][b+1][c  ] += q[i]*hvx[i][j]*(vv[6]/vv[0])*norm;
        jx[a+1][b+1][c  ] += q[i]*hvx[i][j]*(vv[5]/vv[0])*norm;
	
        jx[a  ][b  ][c+1] += q[i]*hvx[i][j]*(vv[3]/vv[0])*norm;
        jx[a+1][b  ][c+1] += q[i]*hvx[i][j]*(vv[4]/vv[0])*norm;
        jx[a  ][b+1][c+1] += q[i]*hvx[i][j]*(vv[2]/vv[0])*norm;
        jx[a+1][b+1][c+1] += q[i]*hvx[i][j]*(vv[1]/vv[0])*norm;
      }
  for(i=0;i<2;i++)
    for(j=0;j<nmx;j++)
      {
        dis_cube(hx[i][j],hy[i][j],hz[i][j],0,hdy,0);
	
        jy[a  ][b  ][c  ] += q[i]*vy[i][j]*(vv[7]/vv[0])*norm;
        jy[a+1][b  ][c  ] += q[i]*vy[i][j]*(vv[8]/vv[0])*norm;
        jy[a  ][b+1][c  ] += q[i]*vy[i][j]*(vv[6]/vv[0])*norm;
        jy[a+1][b+1][c  ] += q[i]*vy[i][j]*(vv[5]/vv[0])*norm;
	
        jy[a  ][b  ][c+1] += q[i]*vy[i][j]*(vv[3]/vv[0])*norm;
        jy[a+1][b  ][c+1] += q[i]*vy[i][j]*(vv[4]/vv[0])*norm;
        jy[a  ][b+1][c+1] += q[i]*vy[i][j]*(vv[2]/vv[0])*norm;
        jy[a+1][b+1][c+1] += q[i]*vy[i][j]*(vv[1]/vv[0])*norm;
      }
  for(i=0;i<2;i++)
    for(j=0;j<nmx;j++)
      {
        dis_cube(hx[i][j],hy[i][j],hz[i][j],0,0,hdz);
	
        jz[a  ][b  ][c  ] += q[i]*vz[i][j]*(vv[7]/vv[0])*norm;
        jz[a+1][b  ][c  ] += q[i]*vz[i][j]*(vv[8]/vv[0])*norm;
        jz[a  ][b+1][c  ] += q[i]*vz[i][j]*(vv[6]/vv[0])*norm;
        jz[a+1][b+1][c  ] += q[i]*vz[i][j]*(vv[5]/vv[0])*norm;
	
        jz[a  ][b  ][c+1] += q[i]*vz[i][j]*(vv[3]/vv[0])*norm;
        jz[a+1][b  ][c+1] += q[i]*vz[i][j]*(vv[4]/vv[0])*norm;
        jz[a  ][b+1][c+1] += q[i]*vz[i][j]*(vv[2]/vv[0])*norm;
        jz[a+1][b+1][c+1] += q[i]*vz[i][j]*(vv[1]/vv[0])*norm;
      }

  /* Boundary conditions of current density */
  for(i=0;i<=imx;i++)
    for(k=0;k<=kmx;k++)
      {
        jy[i][jmx][k] = jy[i][0][k] = jy[i][jmx][k] + jy[i][0][k];
        jy[i][jmp][k] = jy[i][1][k] = jy[i][jmp][k] + jy[i][1][k];
      }
  for(i=0;i<=imx;i++)
    for(j=0;j<=jmx;j++)
      {
        jz[i][j][kmx] = jz[i][j][0] = jz[i][j][kmx] + jz[i][j][0];
        jz[i][j][kmp] = jz[i][j][1] = jz[i][j][kmp] + jz[i][j][1];
      }
  
  for(i=1;i<=imx;i++)
    {
      for(j=0;j<=jmx;j++)
	jx[i][j][kmx] = jx[i][j][0] = jx[i][j][kmx] + jx[i][j][0];
      for(k=0;k<=kmx;k++)
	jx[i][jmx][k] = jx[i][0][k] = jx[i][jmx][k] + jx[i][0][k];
    }
  
  for(i=0;i<=imx;i++)
    for(k=1;k<=kmx;k++)
      {
        jz[i][jmx][k] = jz[i][0][k] = jz[i][jmx][k] + jz[i][0][k];
      }
  
  for(i=0;i<=imx;i++)
    for(j=1;j<=jmx;j++)
      {
        jy[i][j][kmx] = jy[i][j][0] = jy[i][j][kmx] + jy[i][j][0];
      }
}

/*****************************************************************************
*                            Functions of output                             *
*****************************************************************************/
void out_bunch(void)
{
#ifdef energy_total
  out_energy_total();
#endif
#ifdef particle_kinetic
  out_particle_kinetic();
#endif
#ifdef thermalflux
  out_thermalflux();
#endif
#ifdef energy_current
  out_energy_current();
#endif
#ifdef generated_magnetic
  out_generated_magnetic();
#endif
#ifdef particle_statistics
  out_particle_statistics();
#endif
#ifdef particle_lorentz
  out_particle_lorentz();
#endif
#ifdef field_RMS
  if(lap % 20 == 0)
    out_field_RMS();
#endif
#ifdef particle_kinetic
  out_particle_kinetic();
#endif
#ifdef field_absorption
  out_field_absorption();
#endif

  /* functions of hourly */
#ifdef field_magnetic_hourly
  out_field_magnetic_hourly();
#endif
#ifdef field_current_hourly
  out_field_current_hourly();
#endif
#ifdef particle_density_loz_hourly
  out_particle_density_loz_hourly();
#endif
  
  if((lap % STEP == 0 && (int)(lap*dt) >= (int)(BEGIN)) || lap == 0)
    {
#ifdef field_electric
      out_field_electric();
#endif
#ifdef field_magnetic
      out_field_magnetic();
#endif
#ifdef field_current
      out_field_current();
#endif
#ifdef particle_phase
      out_particle_phase();
#endif
#ifdef particle_distribution
      out_particle_distribution();
#endif
#ifdef energy_spectrum
      out_energy_spectrum();
#endif
#ifdef particle_phase_pm3d
      out_particle_phase_pm3d();
#endif
#ifdef particle_density_loz
      out_particle_density_loz();
#endif
#ifdef particle_density_pm3d
      out_particle_density_pm3d();
#endif
#ifdef project_density
      out_project_density();
#endif
#ifdef project_laser
      out_project_laser();
#endif
#ifdef energy_gain
      out_energy_gain();
#endif
    }
}

/************************* Calculation total energy **************************/
#ifdef energy_total
void out_energy_total(void)
{
  double eee[3],bbb[3],kkk[3];
  double ebd[3],bbd[3];
  FILE  *tefp;

  /* Plot the total energy of this simulation */ 
  if(lap == 0)
    tefp = fopen("energy.dat","w");
  else
    tefp = fopen("energy.dat","a");

  for(i=0;i<3;i++)
    eee[i] = bbb[i] = kkk[i] = ebd[i] = bbd[i] = 0.0;
  
  for(i=0;i<2;i++)
    for(j=0;j<nmx;j++)
      kkk[i] += m[i]*(lorentz(vx[i][j],vy[i][j],vz[i][j])-1.0)*norm/tg;

  for(i=0;i<=imx;i++)
    for(j=1;j<=jmx;j++)
      for(k=1;k<=kmx;k++)
        bbb[0] += Bx[i][j][k]*Bx[i][j][k];
  
  for(i=1;i<=imx;i++)
    for(j=0;j<=jmx;j++)
      for(k=1;k<=kmx;k++)
        bbb[1] += By[i][j][k]*By[i][j][k];

  for(i=1;i<=imx;i++)
    for(j=1;j<=jmx;j++)
      for(k=0;k<=kmx;k++)
        bbb[2] += Bz[i][j][k]*Bz[i][j][k];

  for(i=1;i<=imx;i++)
    for(j=0;j<=jmx;j++)
      for(k=0;k<=kmx;k++)
        eee[0] += Ex[i][j][k]*Ex[i][j][k];

  for(i=0;i<=imx;i++)
    for(j=1;j<=jmx;j++)
      for(k=0;k<=kmx;k++)
        eee[1] += Ey[i][j][k]*Ey[i][j][k];

  for(i=0;i<=imx;i++)
    for(j=0;j<=jmx;j++)
      for(k=1;k<=kmx;k++)
        eee[2] += Ez[i][j][k]*Ez[i][j][k];

  /* value on the boundary */
  for(i=1;i<=imx;i++)
    for(k=0;k<=kmx;k++)
      ebd[0] += (Ex[i][0][k]+Ex[i][jmx][k])*(Ex[i][0][k]+Ex[i][jmx][k]);
  for(i=1;i<=imx;i++)
    for(j=0;j<=jmx;j++)
      ebd[0] += (Ex[i][j][0]+Ex[i][j][kmx])*(Ex[i][j][0]+Ex[i][j][kmx]);

  for(i=0;i<=imx;i++)
    for(j=1;j<=jmx;j++)
      ebd[1] += (Ey[i][j][0]+Ey[i][j][kmx])*(Ey[i][j][0]+Ey[i][j][kmx]);

  for(i=0;i<=imx;i++)
    for(k=1;k<=kmx;k++)
      ebd[2] += (Ez[i][0][k]+Ez[i][jmx][k])*(Ez[i][0][k]+Ez[i][jmx][k]);
  
  for(i=1;i<=imx;i++)
    for(k=1;k<=kmx;k++)
      bbd[1] += (By[i][0][k]+By[i][jmx][k])*(By[i][0][k]+By[i][jmx][k]);

  for(i=1;i<=imx;i++)
    for(j=1;j<=jmx;j++)
      bbd[2] += (Bz[i][j][0]+Bz[i][j][kmx])*(Bz[i][j][0]+Bz[i][j][kmx]);

  for(i=0;i<3;i++)
    {
      eee[i] = eee[i]*0.500/tg; 
      bbb[i] = bbb[i]*0.500/tg;
      ebd[i] = ebd[i]*0.125/tg;
      bbd[i] = bbd[i]*0.125/tg;
    }
  
  for(i=0;i<2;i++){
    kkk[i] = kkk[i]*norm/tg;  
  }
  
  for(i=0;i<2;i++)
    kkk[2] += kkk[i];

  for(i=0;i<3;i++)
    kkk[2] += eee[i] + bbb[i] - ebd[i] - bbd[i];;
  
  fprintf(tefp,"%f %e %e %e %e %e %e %e %e %e\n",lap*dt,
	  kkk[0],kkk[1],eee[0]-ebd[0],eee[1]-ebd[1],eee[2]-ebd[2],
	  bbb[0]-bbd[0],bbb[1]-bbd[1],bbb[2]-bbd[2],kkk[2]);
  
  fclose(tefp);
}
#endif

#ifdef particle_kinetic
void out_particle_kinetic(void)
{
  double skx[2],sky[2],skz[2];
  double mvx[2],mvy[2],mvz[2];
  double gma[2];
  FILE *kefp;

  /* Plot the kinetic energy of plasma */
  for(i=0;i<2;i++)
    skx[i] = sky[i] = skz[i] = mvx[i] = mvy[i] = mvz[i] = gma[i] = 0.0;

  for(i=0;i<2;i++)
    for(j=0;j<nmx;j++)
      {	
	mvx[i] += m[i]*vx[i][j];
	mvy[i] += m[i]*vy[i][j];
	mvz[i] += m[i]*vz[i][j];

	skx[i] += m[i]*vx[i][j]*vx[i][j]*0.50;
        sky[i] += m[i]*vy[i][j]*vy[i][j]*0.50;
        skz[i] += m[i]*vz[i][j]*vz[i][j]*0.50;

	gma[i] += lorentz(vx[i][j],vy[i][j],vz[i][j]) - 1.0;
      }

  for(i=0;i<2;i++)
    {
      mvx[i] = mvx[i]*norm/tg;
      mvy[i] = mvy[i]*norm/tg;
      mvz[i] = mvz[i]*norm/tg;

      skx[i] = skx[i]*norm/tg;
      sky[i] = sky[i]*norm/tg;
      skz[i] = skz[i]*norm/tg;

      gma[i] = gma[i]*norm/tg;
    }

  /* file open & writing data */
  if(lap == 0)
    kefp = fopen("kin_ele.dat","w");
  else
    kefp = fopen("kin_ele.dat","a");
  
  fprintf(kefp,"%f %e %e %e %e %e %e %e\n",
	  lap*dt,mvx[0],mvy[0],mvz[0],skx[0],sky[0],skz[0],gma[0]);
  fclose(kefp);

  if(lap == 0)
    kefp = fopen("kin_ion.dat","w");
  else
    kefp = fopen("kin_ion.dat","a");
  
  fprintf(kefp,"%f %e %e %e %e %e %e %e\n",
	  lap*dt,mvx[1],mvy[1],mvz[1],skx[1],sky[1],skz[1],gma[1]);
  fclose(kefp);
}
#endif

#ifdef energy_current
void out_energy_current(void)
{
  double taljp[3],taljs[3];
  FILE *jefp;

  /* Plot the current density energy */
  if(lap == 0)
    jefp = fopen("current.dat","w");
  else
    jefp = fopen("current.dat","a");

  for(i=0;i<3;i++)
    taljp[i] = taljs[i] = 0.0;

  for(i=1;i<imx;i++)
    for(j=0;j<jmx;j++)
      for(k=0;k<kmx;k++)
        taljp[0] += jx[i][j][k];

  for(i=0;i<imx;i++)
    for(j=1;j<jmx;j++)
      for(k=0;k<kmx;k++)
        taljp[1] += jy[i][j][k];

  for(i=0;i<imx;i++)
    for(j=0;j<jmx;j++)
      for(k=1;k<kmx;k++)
        taljp[2] += jz[i][j][k];

  for(i=1;i<=imx;i++)
    for(j=0;j<jmx;j++)
      for(k=0;k<kmx;k++)
        taljs[0] += jx[i][j][k]*jx[i][j][k];

  for(i=0;i<imx;i++)
    for(j=1;j<=jmx;j++)
      for(k=0;k<kmx;k++)
        taljs[1] += jy[i][j][k]*jy[i][j][k];

  for(i=0;i<imx;i++)
    for(j=0;j<jmx;j++)
      for(k=1;k<=kmx;k++)
        taljs[2] += jz[i][j][k]*jz[i][j][k];
  
  for(i=0;i<3;i++){
    taljp[i] = norm*taljp[i]/tg;
    taljs[i] = 0.5*norm*taljs[i]/tg;
  }

  fprintf(jefp,"%e %e %e %e %e %e %e\n",lap*dt,
	  taljs[0],taljs[1],taljs[2],taljp[0],taljp[1],taljp[2]);
  
  fclose(jefp);
}
#endif

/************************** Each physical quantity ***************************/
#ifdef particle_phase
void out_particle_phase(void)
{
  FILE *phfp;
  int pmax = 300000;	

  if(nmx < pmax) pmax = nmx;

  sprintf(fname,"phe%04d.dat",(int)(lap*dt));
  phfp = fopen(fname,"w");
  
  for(j=0;j<pmax;j++)
    fprintf(phfp,"%e %e %e %e %e %e\n",
    x[0][j]/twp,y[0][j]/twp,z[0][j]/twp,vx[0][j],vy[0][j],vz[0][j]);
  
  fclose(phfp);
  
  /*
    sprintf(fname,"phi%04d.dat",(int)(lap*dt));
    phfp = fopen(fname,"w");
  
    for(j=0;j<pmax;j++)
    fprintf(phfp,"%e %e %e %e %e %e\n",
    x[1][j],y[1][j],z[1][j],vx[1][j],vy[1][j],vz[1][j]);
    
    fclose(phfp);
  */
}
#endif

#ifdef thermalflux
void out_thermalflux(void)
{
  double tf1,tf2,tf3;
  double Avx,tal;
  FILE *thfp;

  /* Plot the thermal flux of x direction */
  if(lap == 0)
    thfp = fopen("thermal.dat","w");
  else
    thfp = fopen("thermal.dat","a");

  Avx = 0.0;
  for(j=0;j<nmx;j++)
    if(x[0][j] >= dbp && x[0][j] <= dbp + tlp)
      Avx += vx[0][j]*enz[0][j];
  
  Avx = Avx/(double)(nmx);
  
  tf1 = tf2 = tf3 = 0.0;
  for(j=0;j<nmx;j++)
    if(x[0][j] >= dbp && x[0][j] <= dbp + tlp)
      {
        tal = vav((vx[0][j]*enz[0][j] - Avx),vy[0][j]*enz[0][j],
		  vz[0][j]*enz[0][j]);
	
        tf1 += (vx[0][j]*enz[0][j] - Avx)*tal;
	
        tal  = vav(vx[0][j]*enz[0][j],vy[0][j]*enz[0][j],vz[0][j]*enz[0][j]);
        tf2 += vx[0][j]*enz[0][j]*tal;

        tf3 += (vx[0][j]*enz[0][j])*(vx[0][j]*enz[0][j])*(vx[0][j]*enz[0][j]);
      }
  fprintf(thfp,"%e %e %e %e\n",(lap*dt),tf1,tf2,tf3);

  fclose(thfp);
}
#endif

#ifdef energy_spectrum
#define egd    20000
#define igd    1000
#define MeVe   0.5109990645
#define NC    (4.792038417*1000000*(dx*dy*dz))
#define xegd  (1.0/(double)(egd))

void out_energy_spectrum(void)
{
  double abv[2][nmx];
  double cnte[egd+1],fde[egd+1];
  FILE *esfp;
  /*double cnti[igd+1],fdi[igd+1];*/
  /*double MeVi =  469.3526407;*/
  /*double xigd = 1.0/(double)(igd);*/
  /*FILE *isfp;*/

  sprintf(fname,"spe%04d.dat",(int)(lap*dt));
  esfp = fopen(fname,"w");
  
  for(j=0;j<=egd;j++)
    cnte[j] = 0.0;
  
  for(i=0;i<2;i++)
    for(j=0;j<nmx;j++)
      abv[i][j] = sqrt(vav(vx[i][j],vy[i][j],vz[i][j]));

  for(j=0;j<nmx;j++)
    {
      cnte[(int)(abv[0][j]*(egd))]++;
    }

  for(j=0;j<egd;j++)
    fde[j] = 1.0/sqrt((1.0 - (j*xegd*j*xegd)));
  
  for(i=0;i<egd;i++)
    fprintf(esfp,"%e %e\n",MeVe*(fde[i]-1.0),cnte[i]/NC);
  
  fclose(esfp);
}
#endif

/******************************************************************************
*                            Statistics of particle                           *
******************************************************************************/
#ifdef particle_statistics
void out_particle_statistics(void)
{
  double cnt,tf1,tf2,tal,gvx;
  FILE *avfp,*dvfp,*thfp;

  for(i=0;i<3;i++)
    ave[i] = dev[i] = 0.0;
  
  cnt = tf1 = tf2 = gvx = 0.0;
  
  for(j=0;j<nmx;j++)
    {
      enz[j] = 1.0/sqrt(1.0 - vav(vx[0][j],vy[0][j],vz[0][j]));
      
      ave[0] += vx[0][j];
      ave[1] += vy[0][j];
      ave[2] += vz[0][j];
      gvx    += vx[0][j]*enz[j];

      cnt++;
    }
    
  for(i=0;i<3;i++)
      ave[i] = ave[i]/cnt;
  gvx = gvx/cnt;
  
  for(j=0;j<nmx;j++)
    {
      tal = vav((vx[0][j]*enz[j] - gvx),vy[0][j]*enz[j],vz[0][j]*enz[j]);
      tf1 += (vx[0][j]*enz[j] - gvx)*tal;
      
      tal = vav((vx[0][j] - ave[0]),vy[0][j],vz[0][j]);
      tf2 += (vx[0][j] - ave[0])*tal;
      
      dev[0] += (vx[0][j] - ave[0])*(vx[0][j] - ave[0]);
      dev[1] += (vy[0][j] - ave[1])*(vy[0][j] - ave[1]);
      dev[2] += (vz[0][j] - ave[2])*(vz[0][j] - ave[2]);
    }
  
  for(i=0;i<3;i++)
    dev[i] = sqrt(dev[i]/cnt);
  
  sprintf(fname,"stdevia.dat");
  if(lap == 0)
    dvfp = fopen(fname,"w");
  else
    dvfp = fopen(fname,"a");

  sprintf(fname,"stavera.dat");
  if(lap == 0)
    avfp = fopen(fname,"w");
  else
    avfp = fopen(fname,"a");

  sprintf(fname,"thermal.dat");
  if(lap == 0)
    thfp = fopen(fname,"w");
  else
    thfp = fopen(fname,"a");

  fprintf(avfp,"%f %e %e %e\n",lap*dt,ave[0],ave[1],ave[2]);
  fprintf(dvfp,"%f %e %e %e\n",lap*dt,dev[0],dev[1],dev[2]);
  fprintf(thfp,"%f %e %e\n",lap*dt,tf1,tf2*0.5);

  fclose(avfp);
  fclose(dvfp);
  fclose(thfp);
}
#endif

#ifdef generated_magnetic
void out_generated_magnetic(void)
{
  double bmx[3],bav[3];
  double psx[3],psy[3],psz[3];
  FILE *avfp,*mxfp,*psfp;
  
  if(lap == 0){
    avfp = fopen("sgmfave.dat","w");
    mxfp = fopen("sgmfmax.dat","w");
    psfp = fopen("postmax.dat","w");
  }else{
    avfp = fopen("sgmfave.dat","a");
    mxfp = fopen("sgmfmax.dat","a");
    psfp = fopen("postmax.dat","a");
  }
  
  if(avfp == NULL || mxfp == NULL || psfp == NULL) exit(1);
  
  for(j=0;j<3;j++)
    bmx[j] = bav[j] = 0.0;

  /********** Magnetic field (x) **********/
  for(i=0;i<=imx;i++)
    for(j=1;j<=jmx;j++)
      for(k=1;k<=kmx;k++)
	{
	  bav[0] += fabs(Bx[i][j][k]);
	
	  if(fabs(Bx[i][j][k]) >= bmx[0])
	    {
	      bmx[0] = fabs(Bx[i][j][k]);
	      psx[0] = ((double)(i)*dx    )/twp;
	      psy[0] = ((double)(j)*dy+hdy)/twp;
	      psz[0] = ((double)(k)*dz+hdz)/twp;
	    }
	}
  
  /********** Magnetic field (y) **********/
  for(i=1;i<=imx;i++)
    for(j=0;j<=jmx;j++)
      for(k=1;k<=kmx;k++)
	{
	  bav[1] += fabs(By[i][j][k]);
	  
	  if(fabs(By[i][j][k]) >= bmx[1])
	    {
	      bmx[1] = fabs(By[i][j][k]);
	      psx[1] = ((double)(i)*dx+hdx)/twp;
	      psy[1] = ((double)(j)*dy    )/twp;
	      psz[1] = ((double)(k)*dz+hdz)/twp;
	    }
	}
  
  /********** Magnetic field (z) **********/
  for(i=1;i<=imx;i++)
    for(j=1;j<=jmx;j++)
      for(k=0;k<=kmx;k++)
	{
	  bav[2] += fabs(Bz[i][j][k]);
	  
	  if(fabs(Bz[i][j][k]) >= bmx[2])
	    {
	      bmx[2] = fabs(Bz[i][j][k]);
	      psx[2] = ((double)(i)*dx+hdx)/twp;
	      psy[2] = ((double)(j)*dy+hdy)/twp;
	      psz[2] = ((double)(k)*dz    )/twp;
	    }
	}
  
  for(i=0;i<3;i++)
    bav[i] = bav[i]/tg;
  
  fprintf(avfp,"%f %e %e %e\n",(lap*dt),bav[0],bav[1],bav[2]);
  fprintf(mxfp,"%f %e %e %e\n",(lap*dt),bmx[0],bmx[1],bmx[2]);
  fprintf(psfp,"%e %e %e %e %e %e %e %e %e\n",
	  psx[0],psy[0],psz[0],psx[1],psy[1],psz[1],psx[2],psy[2],psz[2]);
  
  fclose(avfp);
  fclose(mxfp);
  fclose(psfp);
}
#endif

/******************* Electric & magnetic & current field *********************/
#ifdef field_electric
void out_field_electric(void)
{
  FILE *effp;

  /********** Electric field (x,y,z) **********/
  sprintf(fname,"eff%04d.dat",(int)(lap*dt));
  effp = fopen(fname,"w");
  
  for(i=(int)((dbp-um)/dx);i<=(int)((dbp+tlp)/dx);i++)
    for(j=0;j<=jmx;j++)
      for(k=0;k<=kmx;k++)
	fprintf(effp,"%e %e %e\n",
		Ex[i][j][k],Ey[i][j][k],Ez[i][j][k]);  
  fclose(effp);
}
#endif

#ifdef field_magnetic
void out_field_magnetic(void)
{
  FILE *bffp;
 
  /********** Magnetic field (x,y,z) **********/
  sprintf(fname,"bff%04d.dat",(int)(lap*dt));
  bffp = fopen(fname,"w");
  
  for(i=(int)((dbp-um)/dx);i<=(int)((dbp+tlp)/dx);i++)
    for(j=0;j<=jmx;j++)
      for(k=0;k<=kmx;k++)
	fprintf(bffp,"%e %e %e\n",
		Bx[i][j][k],By[i][j][k],Bz[i][j][k]);
  fclose(bffp);
}
#endif

#ifdef field_current
void out_field_current(void)
{
  FILE *jffp;
  
 /********** Current density (x,y,z) **********/
  sprintf(fname,"jff%04d.dat",(int)(lap*dt));
  jffp = fopen(fname,"w");
  
  for(i=(int)((dbp-um)/dx);i<=(int)((dbp+tlp)/dx);i++)
    for(j=0;j<=jmx;j++)
      for(k=0;k<=kmx;k++)
	fprintf(jffp,"%e %e %e\n",
		jx[i][j][k],jy[i][j][k],jz[i][j][k]);
  fclose(jffp);
}
#endif

#ifdef particle_density_loz
void out_particle_density_loz(void)
{
  double dn[imx+2][jmx+2][kmx+2],Loz;
  FILE *dnfp;
  
  sprintf(fname,"dle%04d.dat",(int)(lap*dt));
  dnfp = fopen(fname,"w");
  
  for(i=0;i<=imx;i++)
    for(j=0;j<=jmx;j++)
      for(k=0;k<=kmx;k++)
	dn[i][j][k] = 0.0;
  
  for(j=0;j<nmx;j++)
    {
      Loz = lorentz(vx[0][j],vy[0][j],vz[0][j]) - 1.0;
      dn[nbh(x[0][j],dx)][nbh(y[0][j],dy)][nbh(z[0][j],dz)] += Loz;
    }

  for(i=(int)((dbp-um)/dx);i<=(int)((dbp+tlp)/dx);i++)
    for(j=1;j<=jmx;j++)
      for(k=1;k<=kmx;k++)
	fprintf(dnfp,"%e\n",dn[i][j][k]*norm);
  
  fclose(dnfp);
}
#endif

#ifdef particle_density_pm3d
void out_particle_density_pm3d(void)
{
  double dn[imx+2][jmx+2][kmx+2];
  FILE *dnfp;
  
  sprintf(fname,"dne%04d.dat",(int)(lap*dt));
  dnfp = fopen(fname,"w");
  
  for(i=0;i<=imx;i++)
    for(j=0;j<=jmx;j++)
      for(k=0;k<=kmx;k++)
	dn[i][j][k] = 0.0;
  
  for(j=0;j<nmx;j++)
    dn[nbh(x[0][j],dx)][nbh(y[0][j],dy)][nbh(z[0][j],dz)]++;

  for(i=(int)((dbp-um)/dx);i<=(int)((dbp+tlp)/dx);i++)
    for(j=1;j<=jmx;j++)
      for(k=1;k<=kmx;k++)
	fprintf(dnfp,"%e\n",dn[i][j][k]*norm);
  
  fclose(dnfp);
}
#endif

#ifdef project_laser
void out_project_laser(void)
{
  double aEf[3][imx+1],aBf[3][imx+1];
  double aEt[3][imx+1],aBt[3][imx+1];
  double cyz = jmp * kmp;
  int cyo = (int)(jmx/2);
  int czo = (int)(kmx/2);
  FILE *plfp,*pffp;
  
  sprintf(fname,"prl%04d.dat",(int)(lap*dt));
  plfp = fopen(fname,"w");
  sprintf(fname,"prf%04d.dat",(int)(lap*dt));
  pffp = fopen(fname,"w");

  for(i=0;i<3;i++)
    for(j=0;j<=imx;j++)
      aEf[i][j] = aBf[i][j] = aEt[i][j] = aBt[i][j] = 0.0;
  
  for(i=0;i<=imx;i++)
    for(j=0;j<=jmx;j++)
      for(k=0;k<=kmx;k++)
	{
	  aEf[0][i] += Ex[i][j][k]/cyz;
	  aEf[1][i] += Ey[i][j][k]/cyz;
	  aEf[2][i] += Ez[i][j][k]/cyz;

	  aEt[0][i] += Ex[i][j][k]*Ex[i][j][k]/cyz;
	  aEt[1][i] += Ey[i][j][k]*Ey[i][j][k]/cyz;
	  aEt[2][i] += Ez[i][j][k]*Ez[i][j][k]/cyz;

	  aBf[0][i] += Bx[i][j][k]/cyz;
	  aBf[1][i] += By[i][j][k]/cyz;
	  aBf[2][i] += Bz[i][j][k]/cyz;

	  aBt[0][i] += Bx[i][j][k]*Bx[i][j][k]/cyz;
	  aBt[1][i] += By[i][j][k]*By[i][j][k]/cyz;
	  aBt[2][i] += Bz[i][j][k]*Bz[i][j][k]/cyz;
	}
  
  for(i=0;i<=imx;i++)
    fprintf(plfp,"%e %e %e %e %e %e %e\n",i*dx/twp,
	    Ey[i][cyo][czo],Ex[i][cyo][czo],Ez[i][cyo][czo],
	    Bx[i][cyo][czo],By[i][cyo][czo],Bz[i][cyo][czo]);

  for(i=0;i<=imx;i++)
    fprintf(pffp,"%e %e %e %e %e %e %e %e %e %e %e %e %e\n",i*dx/twp,
	    aEf[0][i],aEf[1][i],aEf[2][i],aBf[0][i],aBf[1][i],aBf[2][i],
	    aEt[0][i],aEt[1][i],aEt[2][i],aBt[0][i],aBt[1][i],aBt[2][i]);
  
  fclose(plfp);
  fclose(pffp);
}
#endif

#ifdef project_density
void out_project_density(void)
{
  double pdn[imx+2];
  FILE *pdfp;
  
  sprintf(fname,"prd%04d.dat",(int)(lap*dt));
  pdfp = fopen(fname,"w");

  for(i=0;i<=imx;i++)
    pdn[i] = 0.0;

  for(j=0;j<nmx;j++)
    pdn[(int)((x[0][j]+hdx)/dx)]++;
  
  for(i=0;i<=imx;i++)
    fprintf(pdfp,"%e %e\n",i*dx/twp,pdn[i]/(double)(jmx*kmx)*norm);

  fclose(pdfp);
}
#endif

#ifdef field_RMS
void out_field_RMS(void)
{
  double Ism,Iyz;
  FILE *rmfp;
  
  if(lap == 0)
    rmfp = fopen("rms_Iyz.dat","w");
  else
    {
      rmfp = fopen("rms_Iyz.dat","a"); 
      fprintf(rmfp,"\n");
    }

  Ism = Iyz = 0.0;
  
  for(i=1;i<=imx;i++)
    for(j=1;j<=jmx;j++)
      for(k=1;k<=kmx;k++)
	Ism += vav(Ex[i][j][k],Ey[i][j][k],Ez[i][j][k])*0.5;
  
  Ism = Ism / (double)(imx*jmx*kmx);
  
  for(i=1;i<=imx;i++)
    {
      Iyz = 0.0;
      
      for(j=1;j<=jmx;j++)
	for(k=1;k<=kmx;k++)
	  Iyz += (Ey[i][j][k]*Ey[i][j][k]+Ez[i][j][k]*Ez[i][j][k])*0.5;

      Iyz = Iyz / (double)(jmx*kmx);
      fprintf(rmfp,"%e %e %e\n",i*dx/twp,lap*dt,
	      sqrt((Ism-Iyz)*(Ism-Iyz)/(double)(jmx*kmx)));
    }

  fclose(rmfp);
}
#endif

#ifdef particle_distribution
#define smx 101
void out_particle_distribution(void)
{
  double cvx[2][smx];
  double cvy[2][smx];
  double cvz[2][smx];
  FILE *dsfp;
   
  sprintf(fname,"pde%04d.dat",(int)(lap*dt));
  dsfp = fopen(fname,"w");

  for(i=0;i<2;i++)
    for(j=0;j<smx;j++)
      cvx[i][j] = cvy[i][j] = cvz[i][j] = 0.0;
  
  for(i=0;i<2;i++)
    for(j=0;j<nmx;j++)
      {
	cvx[i][(int)((vx[i][j]+1)*50)]++;
	cvy[i][(int)((vy[i][j]+1)*50)]++;
	cvz[i][(int)((vz[i][j]+1)*50)]++;
      }
	
  for(i=0;i<2;i++)
    for(j=0;j<smx;j++)
      {
	cvx[i][j] /= (double)nmx;
	cvy[i][j] /= (double)nmx;
	cvz[i][j] /= (double)nmx;
      }

  for(i=0;i<smx;i++)
    fprintf(dsfp,"%e %e %e %e\n",
	    2*(double)i/(double)(smx-1)-1,cvx[0][i],cvy[0][i],cvz[0][i]);

  fclose(dsfp);
}
#endif

#ifdef energy_gain
void out_energy_gain(void)
{
  double vhr[4][imx+1],tpr[4][imx+1],ntr[4][imx+1];
  double cnt[2][imx+1],qth[2][imx+1];
  double Loz,fg,tf1,tf2,avx,avy,avz;
  FILE *egfp,*tpfp;
  
  sprintf(fname,"ege%04d.dat",(int)(lap*dt));
  egfp = fopen(fname,"w");
  sprintf(fname,"tpr%04d.dat",(int)(lap*dt));
  tpfp = fopen(fname,"w");

  avx = avy = avz = 0.0;
  for(i=0;i<=imx;i++)
    {
      cnt[0][i] = cnt[1][i] = qth[0][i] = qth[1][i] = 0.0;
      vhr[0][i] = vhr[1][i] = vhr[2][i] = vhr[3][i] = 0.0;      
    }

  for(j=0;j<nmx;j++)
    {
      avx += vx[0][j]/(double)(nmx);
      avy += vy[0][j]/(double)(nmx);
      avz += vz[0][j]/(double)(nmx);
    }

  for(j=0;j<nmx;j++)
    {	
      Loz = sqrt(1.0/(1.0-vav(vx[0][j],vy[0][j],vz[0][j])));
      tf1 = vav(vx[0][j]*Loz-avx,vy[0][j]*Loz,vz[0][j]*Loz);
      tf2 = vav(vx[0][j]    -avx,vy[0][j]    ,vz[0][j]    );
      
      qth[0][(int)((x[0][j]+hdx)/dx)] += (vx[0][j]*Loz-avx)*tf1;
      qth[1][(int)((x[0][j]+hdx)/dx)] += (vx[0][j]    -avx)*tf2;
      cnt[0][(int)((x[0][j]+hdx)/dx)] ++;
      cnt[1][(int)((x[1][j]+hdx)/dx)] ++;

      vhr[0][(int)((x[0][j]+hdx)/dx)] += (vx[0][j]-avx)*(vx[0][j]-avx);
      vhr[1][(int)((x[0][j]+hdx)/dx)] += (vy[0][j]-avy)*(vy[0][j]-avy);
      vhr[2][(int)((x[0][j]+hdx)/dx)] += (vz[0][j]-avz)*(vz[0][j]-avz);
    }
  
  for(i=0;i<=imx;i++)
    {
      if((int)(cnt[0][i]) == 0)
	{
	  cnt[0][i] = 1.0;
	  fg     = 0.0;
	}else fg = 1.0;
      
      vhr[0][i] = sqrt(vhr[0][i]/cnt[0][i])*fg;
      vhr[1][i] = sqrt(vhr[1][i]/cnt[0][i])*fg;
      vhr[2][i] = sqrt(vhr[2][i]/cnt[0][i])*fg;
      vhr[3][i] = sqrt(vav(vhr[0][i],vhr[1][i],vhr[2][i]));

      tpr[0][i] = vhr[0][i]*vhr[0][i];
      tpr[1][i] = vhr[1][i]*vhr[1][i];
      tpr[2][i] = vhr[2][i]*vhr[2][i];
      tpr[3][i] = vhr[3][i]*vhr[3][i];
    }
  
  for(i=1;i<=imx;i++)
    {
      ntr[0][i] = (tpr[0][i] - tpr[0][i-1])/dx;
      ntr[1][i] = (tpr[1][i] - tpr[1][i-1])/dx;
      ntr[2][i] = (tpr[2][i] - tpr[2][i-1])/dx;
      ntr[3][i] = (tpr[3][i] - tpr[3][i-1])/dx;
    }

  for(i=1;i<=imx;i++)
    fprintf(egfp,"%e %e %e %e %e %e %e %e %e\n",
	    ((i-1)*dx+hdx)/twp,cnt[0][i],cnt[1][i],qth[0][i],
	    qth[1][i],vhr[0][i],vhr[1][i],vhr[2][i],vhr[3][i]);

  for(i=1;i<=imx;i++)
    fprintf(tpfp,"%e %e %e %e %e %e %e %e %e\n",
	    ((i-1)*dx+hdx)/twp,tpr[0][i],tpr[1][i],tpr[2][i],
	    tpr[3][i],ntr[0][i],ntr[1][i],ntr[2][i],ntr[3][i]);
  
  fclose(egfp);
  fclose(tpfp);
}
#endif

#ifdef field_absorption
void out_field_absorption(void)
{
  FILE *abfp;
  double pyx[imx+1],pyy[imx+1];
  double sup[imx+1],sum[imx+1];
  static double tpp[imx+1],tpm[imx+1];
  
  for(i=0;i<=imx;i++)
    sup[i] = sum[i] = 0.0;
  
  for(i=0;i<=imx;i++)
    for(j=1;j<=jmx;j++)
      for(k=1;k<=kmx;j++)
	{
	  pyx[i] = Ey[i][j][k]*Bz[i][j][k];
	  pyy[i] = Ex[i][j][k]*Bz[i][j][k];
	  
	  if(pyx[i] >= 0)
	    sup[i] += sqrt(pyx[i]*pyx[i] + pyy[i]*pyy[i]);
	  else
	    sum[i] += sqrt(pyx[i]*pyx[i] + pyy[i]*pyy[i]);
	}

  for(i=0;i<=imx;i++)
    {
      tpp[i] += sup[i];
      tpm[i] += sum[i];
    }

  if(lap == 0)
    {
      for(i=0;i<=imx;i++)
	tpp[i] = tpm[i] = 0.0;

      abfp = fopen("absorbe.dat","w");
      fclose(abfp);
    }
  
  if(lap % STEP == 0 && lap != 0)
    {
      abfp = fopen("absorbe.dat","a");
      
      for(i=0;i<=imx;i++)
	fprintf(abfp,"%e %e %e %e %e\n",i*dx/twp,lap*dt,
		(1.0-(tpm[i]/(tpp[i]+tpm[i])))*100,tpp[i],tpm[i]);

      fprintf(abfp,"\n");
      fclose(abfp);
    }
}
#endif

/*************************** functions of hourly *****************************/
#ifdef field_magnetic_hourly
#define btu 63.0

void out_field_magnetic_hourly(void)
{
  FILE *bhfp;
  
  if(lap == 0)
    {
      for(i=0;i<=imx;i++)
        for(j=0;j<=jmx;j++)
          for(k=0;k<=kmx;k++)
            Bhx[i][j][k] = Bhy[i][j][k] = Bhz[i][j][k] = 0.0;
    }
  
  /********** Magnetic field (x,y,z) **********/
 if(lap % (int)(btu) == 0 && lap != 0)
   {
     sprintf(fname,"bhf%04d.dat",(int)(lap*dt));
     bhfp = fopen(fname,"w");
     
     for(i=(int)((dbp-um)/dx);i<=(int)((dbp+tlp)/dx);i++)
       for(j=0;j<=jmx;j++)
         for(k=0;k<=kmx;k++)
           fprintf(bhfp,"%e %e %e\n",
                   Bhx[i][j][k]/btu,Bhy[i][j][k]/btu,Bhz[i][j][k]/btu);
     fclose(bhfp);
     
     for(i=0;i<=imx;i++)
        for(j=0;j<=jmx;j++)
          for(k=0;k<=kmx;k++)
            Bhx[i][j][k] = Bhy[i][j][k] = Bhz[i][j][k] = 0.0;
   }

 for(i=0;i<=imx;i++)
   for(j=0;j<=jmx;j++)
     for(k=0;k<=kmx;k++)
       {
	 Bhx[i][j][k] += Bx[i][j][k];
	 Bhy[i][j][k] += By[i][j][k];
	 Bhz[i][j][k] += Bz[i][j][k];
       }
}
#endif

#ifdef field_current_hourly
#define jtu 63.0

void out_field_current_hourly(void)
{
  FILE *jhfp;
  
  if(lap == 0)
    {
      for(i=0;i<=imx;i++)
        for(j=0;j<=jmx;j++)
          for(k=0;k<=kmx;k++)
            jhx[i][j][k] = jhy[i][j][k] = jhz[i][j][k] = 0.0;
    }
  
  /********** current field (x,y,z) **********/
 if(lap % (int)(jtu) == 0 && lap != 0)
   {
     sprintf(fname,"jhf%04d.dat",(int)(lap*dt));
     jhfp = fopen(fname,"w");
     
     for(i=(int)((dbp-um)/dx);i<=(int)((dbp+tlp)/dx);i++)
       for(j=0;j<=jmx;j++)
         for(k=0;k<=kmx;k++)
           fprintf(jhfp,"%e %e %e\n",
                   jhx[i][j][k]/jtu,jhy[i][j][k]/jtu,jhz[i][j][k]/jtu);
     fclose(jhfp);
     
     for(i=0;i<=imx;i++)
        for(j=0;j<=jmx;j++)
          for(k=0;k<=kmx;k++)
            jhx[i][j][k] = jhy[i][j][k] = jhz[i][j][k] = 0.0;
   }

 for(i=0;i<=imx;i++)
   for(j=0;j<=jmx;j++)
     for(k=0;k<=kmx;k++)
       {
	 jhx[i][j][k] += jx[i][j][k];
	 jhy[i][j][k] += jy[i][j][k];
	 jhz[i][j][k] += jz[i][j][k];
       }
}
#endif

#ifdef particle_density_loz_hourly
#define dtu 63.0

void out_particle_density_loz_hourly(void)
{
  static double dnh[imx+2][jmx+2][kmx+2];
  double Loz;
  FILE *dnfp;

  if(lap == 0)
    {
      for(i=0;i<=imx;i++)
	for(j=0;j<=jmx;j++)
	  for(k=0;k<=kmx;k++)
	    dnh[i][j][k] = 0.0;
    }
  
  if(lap % (int)(jtu) == 0 && lap != 0)
     {
       sprintf(fname,"dlh%04d.dat",(int)(lap*dt));
       dnfp = fopen(fname,"w");
       
       for(i=(int)((dbp-um)/dx);i<=(int)((dbp+tlp)/dx);i++)
	 for(j=0;j<=jmx;j++)
	   for(k=0;k<=kmx;k++)
	     fprintf(dnfp,"%e\n",dnh[i][j][k]/dtu*norm);
       
       fclose(dnfp);
     }

   for(j=0;j<nmx;j++)
     {
       Loz = lorentz(vx[0][j],vy[0][j],vz[0][j]) - 1.0;
       dnh[nbh(x[0][j],dx)][nbh(y[0][j],dy)][nbh(z[0][j],dz)] += Loz;
     }
}
#endif

/*************************** Simulation of setup *****************************/
#ifdef system_setup
void out_system_setup(void)
{
  FILE *stfp;

  /********** Plot the setup file of this simulation **********/
  sprintf(fname,"setup.dat");
  stfp = fopen(fname,"w");

  fprintf(stfp,"Setup of 3D PIC Simulation ver 1.3 \n");
  fprintf(stfp,"X grid spacing           : dx    = %f\n",     dx);
  fprintf(stfp,"Y grid spacing           : dy    = %f\n",     dy);
  fprintf(stfp,"Z grid spacing           : dz    = %f\n",     dz);
  fprintf(stfp,"Time Step                : dt    = %f\n",     dt);
  fprintf(stfp,"X grid cells             : imx   = %d\n",    imx);
  fprintf(stfp,"Y grid cells             : jmx   = %d\n",    jmx);
  fprintf(stfp,"Z grid cells             : kmx   = %d\n",    kmx);
  fprintf(stfp,"Distance of X            : Lx    = %f\n",     Lx);
  fprintf(stfp,"Distance of Y            : Ly    = %f\n",     Ly);
  fprintf(stfp,"Distance of Z            : Lz    = %f\n",     Lz);
  fprintf(stfp,"Number of cell           : tg    = %d\n",     tg);
  fprintf(stfp,"Absorption const         : dt_dx = %f\n",dt_dx(dt));
  fprintf(stfp,"Time Limit (lap)         : tmx   = %d\n",    tmx);
  fprintf(stfp,"Number of particle       : nmx   = %d\n",    nmx);
  fprintf(stfp,"Thermal Velocity(x) of e : vtxe  = %f\n", vtx[0]);
  fprintf(stfp,"Thermal Velocity(y) of e : vtye  = %f\n", vty[0]);
  fprintf(stfp,"Thermal Velocity(z) of e : vtze  = %f\n", vtz[0]);
  fprintf(stfp,"Thermal Velocity(x) of i : vtxi  = %f\n", vtx[1]);
  fprintf(stfp,"Thermal Velocity(y) of i : vtyi  = %f\n", vty[1]);
  fprintf(stfp,"Thermal Velocity(z) of i : vtzi  = %f\n", vtz[1]);
  fprintf(stfp,"Drift Velocity(x) of e   : vdxe  = %f\n", vdx[0]);
  fprintf(stfp,"Drift Velocity(y) of e   : vdye  = %f\n", vdy[0]);
  fprintf(stfp,"Drift Velocity(z) of e   : vdze  = %f\n", vdz[0]);
  fprintf(stfp,"Drift Velocity(x) of i   : vdxi  = %f\n", vdx[1]);
  fprintf(stfp,"Drift Velocity(y) of i   : vdyi  = %f\n", vdy[1]);
  fprintf(stfp,"Drift Velocity(z) of i   : vdzi  = %f\n", vdz[1]);
  fprintf(stfp,"\n");

  fprintf(stfp,"Paramerter of LASER     \n");
  fprintf(stfp,"Intensity of Laser       : Eo    = %f\n",     Eo);
  fprintf(stfp,"A constant time          : tau   = %f\n",    tau);
  fprintf(stfp,"Width of Laser           : w     = %f\n",      w);
  fprintf(stfp,"Laser life time (lap)    : llt   = %d\n",(int)llt);
  fprintf(stfp,"Laser life time (double) : llt   = %f\n",    llt);
  fprintf(stfp,"\n");

  fprintf(stfp,"Parameter of Plasma wall\n");
  fprintf(stfp,"Total layer plasma wakk  : tlp   = %f\n",    tlp);
#ifdef gradient
  fprintf(stfp,"Layer plasma wall        : lp    = %f\n",     lp);
  fprintf(stfp,"Gradient Layer wall      : lg    = %f\n",     lg);
#else
  fprintf(stfp,"Layer plasma wall        : lp    = %f\n",     lp);
#endif
  fprintf(stfp,"Distance by plasma wall  : dbp   = %f\n",    dbp);
  fprintf(stfp,"Clitical density ratio   : no    = %f\n",     no);
  fprintf(stfp,"Particles by unit cell   : np    = %e\n",     np);
  fprintf(stfp,"Standardization constant : norm  = %e\n",   norm);
  fprintf(stfp,"\n");

  /***************** Check errors of simulation parameter *******************/
  i = 0;
  fprintf(stfp,"Error Check \n");
  if(Lx < (dbp+tlp)){
    fprintf(stfp,"too short Lx or huge dbp & tlp ...\n");
    i++;
  }
  if(fabs((double)(llt-(3.141592*tau/dt))) > 5.0){
    fprintf(stfp,"no much irradiation time of LASER ...\n");
    fprintf(stfp,"llt : lap = %d : %d \n",(int)llt,(int)(3.141592*tau/dt));
    i++;
  }
  if(Lx < (dbp+tlp)){
    fprintf(stfp,"too much bfp ...\n");
    fprintf(stfp,"Lx : dbp+tlp = %f:%f ...\n",Lx,dbp+tlp);
    i++;
  }
  
  fprintf(stfp,"\ntotal error items (%d)\n",i);
  fprintf(stfp,"\n");
  fclose(stfp);
}
#endif

/*****************************************************************************
*                             Utility Functions                              *
*****************************************************************************/
double grad(double width,double adside)
{
  double rx,ry;
  
  for(;;)
    {
      rx = rnd(1);
      ry = rnd(1);
      if(rx > ry) break;
    }

  return width*rx + adside;
}

double vav(double tx,double ty,double tz)
{
  return ((tx*tx)+(ty*ty)+(tz*tz));
}

double rnd(double tx)
{
  return tx*rand()/(RAND_MAX+1.0);
}

double lorentz(double tx,double ty,double tz)
{
  return sqrt(1.0/(1.0 - vav(tx,ty,tz)));
}

double dt_dx(double tx)
{
  return (tx - dx)/(tx + dx);
}

int nbh(double tr,double ddr)
{
  return (tr + (ddr/2.0))/ddr;
}

/*****************************************************************************
*                    Laser Plasma Interaction Simulator                      *
*                                    Version 1.0a                            *
*                                       2005.9.12                            *
*                          Copyright  Keita Ogawa                            * 
*****************************************************************************/
